package net.wanji.web.controller;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import net.wanji.web.common.Result;
import net.wanji.web.util.HttpRestUtil;
import net.wanji.web.vo.signalservice.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/04 16:10
 */
@Api(value = "SignalServiceController", description = "信号控制")
@RequestMapping("/controlCommand")
@RestController
@SuppressWarnings("all")
public class SignalServiceController {

    @Autowired
    private HttpRestUtil httpRestUtil;

    @Value("${utcServiceUrl}")
    private String utcServiceUrl;

    @ApiOperation(value = "信号机方案下发-基础方案下发", notes = "信号机方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public <T> Result<T> schemeSend(HttpServletRequest httpServletRequest, @RequestBody SchemeSendVO schemeSendVO) {
        String param = JSONObject.toJSONString(schemeSendVO);
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public <T> Result<T> planSend(HttpServletRequest httpServletRequest, @RequestBody PlanSendVO planSendVO) throws Exception {
        String param = JSONObject.toJSONString(planSendVO);
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    @ApiOperation(value = "时间表下发-时间表信息", notes = "时间表下发-时间表信息")
    @PostMapping("/scheduleSend")
    public <T> Result<T> scheduleSend(HttpServletRequest httpServletRequest, @RequestBody ScheduleSendVO scheduleSendVO) throws Exception {
        String param = JSONObject.toJSONString(scheduleSendVO);
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    @ApiOperation(value = "相位配时下发-（下发相位参数）", notes = "相位配时下发-（下发相位参数）")
    @PostMapping("/phaseTimingSend")
    public <T> Result<T> phaseTimingSend(HttpServletRequest httpServletRequest, @RequestBody PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        String param = JSONObject.toJSONString(phaseTimingSendVO);
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    @ApiOperation(value = "灯态控制-路口全红/黄闪/关灯控制/恢复", notes = "灯态控制-路口全红/黄闪/关灯控制/恢复")
    @PostMapping("/lampStateControl")
    public <T> Result<T> allRedControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        Integer controlType = commandVO.getControlType();
        String uri = "";
        if (1 == controlType) {
            uri = "/controlCommand/allRedControl";
        }
        if (2 == controlType) {
            uri = "/controlCommand/yellowLightControl";
        }
        if (3 == controlType) {
            uri = "/controlCommand/closeLightControl";
        }
        String param = JSONObject.toJSONString(commandVO);
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(uri), null, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(uri)));
        }
        return Result.success("success");
    }


    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "command", value = "1 开始步进 0 取消步进", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = "stepNum", value = "0 顺序步进 n 跳过n个相位", paramType = "query", required = true, dataType = "int"),
    })
    @PostMapping("/stepControl")
    public <T> Result<T> stepControl(HttpServletRequest httpServletRequest, @RequestParam String code,
                                     @RequestParam Integer command,
                                     @RequestParam int stepNum) throws Exception {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("code", code);
        map.put("command", command);
        map.put("stepNum", stepNum);
        String param = JSONObject.toJSONString(map);
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    @ApiOperation(value = "恢复时间表-恢复路口时间表执行", notes = "恢复时间表-恢复路口时间表执行")
    @PostMapping("/recoverSchedule")
    public <T> Result<T> recoverSchedule(HttpServletRequest httpServletRequest, @RequestParam String code) throws Exception {
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, code, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public <T> Result<T> lockControl(HttpServletRequest httpServletRequest, @RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        String param = JSONObject.toJSONString(commandVO);
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    @ApiOperation(value = "相位差下发", notes = "相位差下发")
    @PostMapping("/phaseDiffSend")
    public <T> Result<T> phaseDiffSend(HttpServletRequest httpServletRequest) {
        Result result = httpRestUtil.doPost(utcServiceUrl.concat(getRemoteUri(httpServletRequest)), null, null, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "远程服务调用异常url".concat(utcServiceUrl.concat(getRemoteUri(httpServletRequest))));
        }
        return Result.success("success");
    }

    public static String getRemoteUri(HttpServletRequest httpServletRequest) {
        String requestURI = httpServletRequest.getRequestURI().substring(4);
        return requestURI;
    }
}
